/*
 * Copyright (c) 2010, Duncan Barclay, dmlb@dmlb.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. The names of its contributors
 * may not be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id$
 * 
 */

using System;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using HttpServer;
using Microsoft.MediaCenter;
using Microsoft.MediaCenter.Hosting;
using Microsoft.MediaCenter.UI;

namespace webmc
{
	public class BackgroundApplication : ModelItem
	{
		private AddInHost m_current = null;
		private WMPLib.WindowsMediaPlayer m_player = null;

		public BackgroundApplication(AddInHost host)
		{
			this.m_current = host;
		}

		public AddInHost Current
		{
			get
			{
				return m_current;
			}
		}

		public WMPLib.WindowsMediaPlayer Player
		{
			get
			{
				if (m_player == null)
				{
					try
					{
						m_player = new WMPLib.WindowsMediaPlayer();
					}
					catch (Exception ex)
					{
						throw new HttpException(System.Net.HttpStatusCode.InternalServerError, "WindowsMediaPlayer failed: " + ex.ToString());
					}
				}
				return m_player;
			}
		}

		public ApplicationContext ApplicationContext
		{
			get
			{
				if (Current == null)
				{
					return null;
				}
				return Current.ApplicationContext;
			}
		}

		public MediaCenterEnvironment MediaCenterEnvironment
		{
			get
			{
				if (Current == null)
				{
					return null;
				}
				return Current.MediaCenterEnvironment;
			}
		}

		// http://discuss.mediacentersandbox.com/forums/thread/9123.aspx
		public void _ResetAddInHostMediaExperienceCache()
		{
			if (true) // Was IsWindows7...
			{
				try
				{
					FieldInfo fi_single = AddInHost.Current.MediaCenterEnvironment.GetType().GetField("_checkedMediaExperience", BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance);

					if (fi_single != null)
					{
						fi_single.SetValue(AddInHost.Current.MediaCenterEnvironment, false);
					}
					else
					{
						throw new HttpException(System.Net.HttpStatusCode.InternalServerError, "Warning: cannot find MediaCenterEnvironment._checkedMediaExperience field on Windows 7");
					}
				}
				catch (Exception e)
				{
					throw new HttpException(System.Net.HttpStatusCode.InternalServerError, "Error setting MediaCenterEnvironment._checkedMediaExperience flag on Windows 7: " + e.ToString());
				}
			}
		}

		public void _ResetWebmcMediaExperienceCache()
		{
			if (true) // Was IsWindows7...
			{
				try
				{
					FieldInfo fi_single = Current.MediaCenterEnvironment.GetType().GetField("_checkedMediaExperience", BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance);

					if (fi_single != null)
					{
						fi_single.SetValue(Current.MediaCenterEnvironment, false);
					}
					else
					{
						throw new HttpException(System.Net.HttpStatusCode.InternalServerError, "Warning: cannot find MediaCenterEnvironment._checkedMediaExperience field on Windows 7");
					}
				}
				catch (Exception e)
				{
					throw new HttpException(System.Net.HttpStatusCode.InternalServerError, "Error setting MediaCenterEnvironment._checkedMediaExperience flag on Windows 7: " + e.ToString());
				}
			}
		}

		public AudioMixer AudioMixer
		{
			get
			{
				if (MediaCenterEnvironment == null)
				{
					return null;
				}
				return MediaCenterEnvironment.AudioMixer;
			}
		}

		public MediaExperience MediaExperience
		{
			get
			{
				if (MediaCenterEnvironment == null)
				{
					return null;
				}
				// The MediaExperience object is marshalled,and marshalled
				// objects have timeout on them, this attempts to reset
				// the object if it has gone away - note the MediaExperience
				// object is not always there legitimately, i.e. when not
				// playing anything, so we just do this once to try and
				// get it back
				if (Current.MediaCenterEnvironment.MediaExperience == null)
				{
					_ResetWebmcMediaExperienceCache();
				}
				return MediaCenterEnvironment.MediaExperience;
			}
		}

		public MediaMetadata MediaMetadata
		{
			get
			{
				if (MediaExperience == null)
				{
					return null;
				}
				return MediaExperience.MediaMetadata;
			}
		}

		public MediaTransport MediaTransport
		{
			get
			{
				if (MediaExperience == null)
				{
					return null;
				}
				return MediaExperience.Transport;
			}
		}

		public void Start()
		{
			string temp = "The background application did something. " + Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData);
			DialogTest(temp);
		}

		public void DialogTest(string strClickedText)
		{
			int timeout = 5;
			bool modal = false;
			string caption = Resources.DialogCaption;

			if (Current != null)
			{
				MediaCenterEnvironment.Dialog(strClickedText,
											  caption,
											  new object[] { DialogButtons.Ok },
											  timeout,
											  modal,
											  null,
											  delegate(DialogResult dialogResult) { });
			}
			else
			{
				Debug.WriteLine("DialogTest");
			}
		}
	}
}