/*
 * Copyright (c) 2010, Duncan Barclay, dmlb@dmlb.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. The names of its contributors
 * may not be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id$
 * 
 */

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Threading;
using HttpServer;
using HttpServer.Modules;
using HttpServer.Resources;
using Microsoft.MediaCenter.Hosting;
using HttpListener = HttpServer.HttpListener;

namespace webmc
{
	public class WebmcBackgroundAddIn : IAddInModule, IAddInEntryPoint
	{
		/// <summary>
		/// Used to signal that the plug-in should exit
		/// </summary>
		private EventWaitHandle m_waitHandle = new ManualResetEvent(false);

		/// <summary>
		/// HTTP Listener
		/// </summary>
		private Server m_httpServer;

		/// <summary>
		/// Application Container
		/// </summary>
		static public BackgroundApplication Host;

		/// <summary>
		/// List of modules used in the server
		/// </summary>
		private IModule[] m_modules;

		/// <param name="appInfo"></param>
		/// <param name="entryPointInfo"></param>
		public void Initialize(Dictionary<string, object> appInfo, Dictionary<string, object> entryPointInfo)
		{
			//  Sets the wait handle to the launch method will not exit
			m_waitHandle.Reset();
		}

		public void Uninitialize()
		{
			//  Allow our launch method to exit
			m_waitHandle.Set();
		}

		public void Launch(AddInHost host)
		{
			if (host != null && host.ApplicationContext != null)
			{
				host.ApplicationContext.SingleInstance = true;
			}
			Host = new BackgroundApplication(host);
			try
			{
				//  Lower the priority of this thread
				System.Threading.Thread.CurrentThread.Priority = ThreadPriority.Lowest;

				m_httpServer = new Server();

				// use one http listener.
				m_httpServer.Add(HttpListener.Create(IPAddress.Any, 40960));

				// add our own modules
				m_modules = new IModule[] {
					new Module_MediaMetaData(),
					new Module_Artwork(),
					new Module_Albums(),
					new Module_AlbumTracks(),
					new Module_Images(),
					new Module_Play(),
					new Module_Playlists(),
					new Module_PlayRate(),
					new Module_Queue(),
					new Module_SoundSwitch(),
					new Module_DisplaySwitch(),
					new Module_Tracks(),
					new Module_UnknownAlbums(),
					new Module_Volume()
				};
				for (var moduleIndex = 0; moduleIndex < m_modules.Length; moduleIndex++)
				{
					m_httpServer.Add(m_modules[moduleIndex]);
				}

				// add file handling using a module provided with HttpServer
				ResourceProvider resources = new ResourceProvider();
				resources.Add(new FileResources("/files", Webmc.HtDocsDir));
				m_httpServer.Add(new FileModule(resources, false));

				// add missing page module
				m_httpServer.Add(new Module_MissingPage(m_modules));

				// start server, can have max 5 pending accepts.
				m_httpServer.Start(5);

				//  Wait until exit request from host
				m_waitHandle.WaitOne();
			}
			catch (Exception ex)
			{
				Trace.TraceError(ex.ToString());
			}
		}
	}

	public class WebmcForegroundAddIn0 : IAddInModule, IAddInEntryPoint
	{
		private static HistoryOrientedPageSession s_session;

		public void Initialize(Dictionary<string, object> appInfo, Dictionary<string, object> entryPointInfo)
		{
		}

		public void Uninitialize()
		{
		}

		public void Launch(AddInHost host)
		{
			if (host != null && host.ApplicationContext != null)
			{
				host.ApplicationContext.SingleInstance = true;
			}
			s_session = new HistoryOrientedPageSession();
			ForegroundApplication app = new ForegroundApplication(s_session, host);
			app.GoToMenu();
		}

	}

	public class WebmcForegroundAddIn1 : IAddInModule, IAddInEntryPoint
	{

		public void Initialize(Dictionary<string, object> appInfo, Dictionary<string, object> entryPointInfo)
		{
		}

		public void Uninitialize()
		{
		}

		public void Launch(AddInHost host)
		{
			if (host != null && host.ApplicationContext != null)
			{
				host.ApplicationContext.SingleInstance = true;
			}
			String binaryPath;
			binaryPath = Path.Combine(Webmc.BinDir, "setaudio_itemn.exe");
			System.Diagnostics.Process process = new System.Diagnostics.Process();
			process.StartInfo.FileName = binaryPath;
			process.StartInfo.Arguments = "1";
			process.Start();
			process.WaitForExit();
		}
	}

	public class WebmcForegroundAddIn2 : IAddInModule, IAddInEntryPoint
	{
		public void Initialize(Dictionary<string, object> appInfo, Dictionary<string, object> entryPointInfo)
		{
		}

		public void Uninitialize()
		{
		}

		public void Launch(AddInHost host)
		{
			if (host != null && host.ApplicationContext != null)
			{
				host.ApplicationContext.SingleInstance = true;
			}
			String binaryPath;
			binaryPath = Path.Combine(Webmc.BinDir, "setaudio_itemn.exe");
			System.Diagnostics.Process process = new System.Diagnostics.Process();
			process.StartInfo.FileName = binaryPath;
			process.StartInfo.Arguments = "2";
			process.Start();
			process.WaitForExit();
		}
	}
}