﻿/*
 * Copyright (c) 2010, Duncan Barclay, dmlb@dmlb.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. The names of its contributors
 * may not be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id$
 * 
 */

using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using HttpServer;
using HttpServer.Modules;
using Newtonsoft.Json.Linq;

namespace webmc
{
	class Module_Artwork : IModule, IWebmcModule
	{
		public String MYPATH
		{
			get
			{
				return "/artwork/";
			}
		}
		public String USAGE
		{
			get
			{
				return MYPATH + "TrackingID";
			}
		}

		public ProcessingResult Process(RequestContext context)
		{
			if (!context.Request.Uri.AbsolutePath.StartsWith(MYPATH))
			{
				return ProcessingResult.Continue;
			}

			try
			{
				JObject json = new JObject();

				WMPLib.IWMPMediaCollection2 collection = (WMPLib.IWMPMediaCollection2)WebmcBackgroundAddIn.Host.Player.mediaCollection;
				WMPLib.IWMPQuery query;
				WMPLib.IWMPStringCollection sourceURL;

				String param = Path.GetFileNameWithoutExtension(context.Request.Uri.AbsolutePath);
				if (param.Length == 0)
				{
					throw new HttpException(System.Net.HttpStatusCode.BadRequest, USAGE);
				}
#if DEBUG
				json.Add("x-param", param);
				json.Add("x-param-unescaped", Uri.UnescapeDataString(param));
#endif

				//
				// Look up the track by its WMContentID, and use the sourceURL attribute as a base to find image files
				//
				query = collection.createQuery();
				query.addCondition("TrackingID", "Equals", Uri.UnescapeDataString(param));
				try
				{
					sourceURL = collection.getStringCollectionByQuery("SourceURL", query, "Audio", "Artist", true);
				}
				catch (Exception ex)
				{
					throw new HttpException(System.Net.HttpStatusCode.BadRequest, "Query failed using WMContentID " + param + "<br />" + ex.ToString());
				}
				if (sourceURL.count == 0)
				{
					throw new HttpException(System.Net.HttpStatusCode.BadRequest, "No item with TrackingID " + param);
				}
				if (sourceURL.count > 1)
				{
					throw new HttpException(System.Net.HttpStatusCode.BadRequest, "Multiple items with TrackingID " + param);
				}

				String path = Path.GetDirectoryName(sourceURL.Item(0));
				String imagePath;
				byte[] buffer;
				Boolean imageFound = false;

				// Try Folder.jpg
				imagePath = path + "\\Folder.jpg";
				if (!imageFound && File.Exists(imagePath) && _loadImage(imagePath, out buffer))
				{
#if DEBUG
					json.Add("x-imagePath", imagePath);
#endif
					imageFound = true;
					json.Add("image", buffer);
				}

				// Try AlbumArtSmall.jpg
				imagePath = path + "\\AlbumArtSmall.jpg";
				if (!imageFound && File.Exists(imagePath) && _loadImage(imagePath, out buffer))
				{
#if DEBUG
					json.Add("x-imagePath", imagePath);
#endif
					imageFound = true;
					json.Add("image", buffer);
				}

				// Generate a default image if we haven't found one already
				if (!imageFound) {
					Bitmap bitmap = Resources.album_missing;
					String albumTitle;

					sourceURL = collection.getStringCollectionByQuery("WM/AlbumTitle", query, "Audio", "Artist", true);
					albumTitle = sourceURL.Item(0);

					using (Graphics graphics = Graphics.FromImage(bitmap))
					{
						RectangleF drawRectangle = new RectangleF(10, 10, bitmap.Width - 10, bitmap.Height - 10);
						using (Font drawFont = new Font("Arial", 16))
						{
							// Draw the text into a wrapped rectangle, with a shadow
							drawRectangle.Offset(+1, +1);
							graphics.DrawString(albumTitle.ToString(), drawFont, Brushes.White, drawRectangle);
							drawRectangle.Offset(-1, -1);
							graphics.DrawString(albumTitle.ToString(), drawFont, Brushes.Black, drawRectangle);
						}
					}

					// Copy the generated image into the buffer
					MemoryStream imageMs = new MemoryStream();
					int read;
					bitmap.Save(imageMs, ImageFormat.Png);
					imageMs.Seek(0, SeekOrigin.Begin);
					buffer = new byte[imageMs.Length];
					read = imageMs.Read(buffer, 0, buffer.Length);
					if (read != buffer.Length)
					{
						throw new Exception("short read");
					}
#if DEBUG
					json.Add("x-imagePath", "not found");
					json.Add("x-albumTitle", albumTitle.ToString());
					json.Add("x-imageMs.Length", imageMs.Length);
					json.Add("x-buffer.Length", buffer.Length);
					json.Add("x-read", read);
#endif
					json.Add("image", buffer);
				}

				json.Add(context.Request.Uri.AbsolutePath, true.ToString());
				return OutputData.Output(context, json);
			}
			catch (HttpException ex)
			{
				throw ex;
			}
			catch (Exception ex)
			{
				return OutputException.Output(context, ex);
			}
		}

		private bool _loadImage(String imagePath, out byte[] buffer)
		{
			FileStream imageFs;
			int read;
			try
			{
				imageFs = new FileStream(imagePath, FileMode.Open, FileAccess.Read, FileShare.Read);
				buffer = new byte[imageFs.Length];
				try
				{
					read = imageFs.Read(buffer, 0, buffer.Length);
					if (read != buffer.Length)
					{
						throw new Exception("short read");
					}
				}
				catch (Exception ex)
				{
					throw new HttpException(System.Net.HttpStatusCode.BadRequest, "Error reading image file: " + imagePath + "<br />" + ex.ToString());
				}
				finally
				{
					imageFs.Close();
				}
			}
			catch (HttpException ex)
			{
				throw ex;
			}
			catch (Exception ex)
			{
				throw new HttpException(System.Net.HttpStatusCode.BadRequest, "Error opening image file: " + "<br />" + ex.ToString());
			}
			return true;
		}
	}
}