﻿/*
 * Copyright (c) 2010, Duncan Barclay, dmlb@dmlb.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. The names of its contributors
 * may not be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id$
 * 
 */

using System;
using System.Collections.Generic;
using System.IO;
using HttpServer;
using HttpServer.Modules;
using Newtonsoft.Json.Linq;

namespace webmc
{
	class Module_Tracks : IModule, IWebmcModule
	{
		public String MYPATH
		{
			get
			{
				return "/tracks/";
			}
		}
		public String USAGE
		{
			get
			{
				return MYPATH + "WMContentID|WMCollectionGroupID|TrackingID";
			}
		}

		private Dictionary<String, Int32> m_trackMediaAtoms = new Dictionary<String, Int32>();

		public Module_Tracks()
		{
			WMPLib.IWMPMediaCollection2 collection = (WMPLib.IWMPMediaCollection2)WebmcBackgroundAddIn.Host.Player.mediaCollection;
			String[] attributes;

			// Build a list of MediaAtoms for efficicency when looking up track info
			attributes = new[] {
					"Title",
					"TrackingID",
					"WM/AlbumArtist",
					"WM/AlbumTitle",
					"WM/Genre",
					"WM/WMCollectionID",
					"WM/WMCollectionGroupID",
					"WM/WMContentID",
					"WM/TrackNumber",
				};
			foreach (String attribute in attributes)
			{
				m_trackMediaAtoms.Add(attribute, collection.getMediaAtom(attribute));
			}
		}

		public ProcessingResult Process(RequestContext context)
		{
			if (!context.Request.Uri.AbsolutePath.StartsWith(MYPATH))
			{
				return ProcessingResult.Continue;
			}

			try
			{
				JObject json = new JObject();
				JArray jsonTracks = new JArray();
				JObject jsonTrack;

				WMPLib.IWMPMediaCollection2 mediaCollection = (WMPLib.IWMPMediaCollection2)WebmcBackgroundAddIn.Host.Player.mediaCollection;
				WMPLib.IWMPPlaylistCollection playlistCollection = WebmcBackgroundAddIn.Host.Player.playlistCollection;
				WMPLib.IWMPQuery query;
				WMPLib.IWMPPlaylist tracks = null;
				WMPLib.IWMPPlaylist playlists;
				WMPLib.IWMPMedia track;
				WMPLib.IWMPMedia playlist;

				String param = Path.GetFileNameWithoutExtension(context.Request.Uri.AbsolutePath);
				if (param.Length == 0)
				{
					throw new HttpException(System.Net.HttpStatusCode.BadRequest, USAGE);
				}
#if DEBUG
				json.Add("x-param", param);
				json.Add("x-param-unescaped", Uri.UnescapeDataString(param));
#endif
				if (mediaCollection == null)
				{
					throw new HttpException(System.Net.HttpStatusCode.InternalServerError, "No mediaCollection");
				}
				else if (playlistCollection == null)
				{
					throw new HttpException(System.Net.HttpStatusCode.InternalServerError, "No playlistCollection");
				}
				else
				{
					//
					// Look up the content by its WMContentID, WMCollectionGroupID, TrackingID to find the tracks
					//
					String[][] conditions = new[] { 
						new[] {"WM/WMContentID", "Audio"},
						new[] {"WM/WMCollectionGroupID", "Audio"},
						new[] {"TrackingID", "Audio"},
						new[] {"TrackingID", "Playlist"}
					};
					foreach (String[] condition in conditions)
					{
						try
						{
							if (condition[1] == "Audio")
							{
								query = mediaCollection.createQuery();
								query.addCondition(condition[0], "Equals", Uri.UnescapeDataString(param));
								tracks = mediaCollection.getPlaylistByQuery(query, condition[1], "WM/TrackNumber", true);
							}
							else if (condition[1] == "Playlist")
							{
								query = mediaCollection.createQuery();
								query.addCondition(condition[0], "Equals", Uri.UnescapeDataString(param));
								playlists = mediaCollection.getPlaylistByQuery(query, condition[1], "Title", true);
								playlist = playlists.get_Item(0);
								tracks = playlistCollection.getByName(playlist.name).Item(0);
							}
						}
						catch (Exception ex)
						{
							throw new HttpException(System.Net.HttpStatusCode.BadRequest, "Query failed using GUID " + Uri.UnescapeDataString(param) + "<br />" + ex.ToString());
						}
						if (tracks.count != 0)
						{
							break;
						}
					}
					if (tracks == null || tracks.count == 0)
					{
						throw new HttpException(System.Net.HttpStatusCode.BadRequest, "No item with GUID " + Uri.UnescapeDataString(param));
					}

					// Loop over the tracks and create an array, filling in key details
					jsonTracks = new JArray();
					for (int trackIndex = 0; trackIndex < tracks.count; trackIndex++)
					{
						track = tracks.get_Item(trackIndex);
						jsonTrack = new JObject();
#if DEBUG
						String attrName, attrValue;
						for (int trackAttrIndex = 0; trackAttrIndex < track.attributeCount; trackAttrIndex++)
						{
							attrName = track.getAttributeName(trackAttrIndex);
							attrValue = track.getItemInfo(attrName);
							jsonTrack.Add("Looped-"+attrName.ToString(), attrValue.ToString());
						}
#endif
						foreach (KeyValuePair<String, Int32> atom in m_trackMediaAtoms)
						{
							jsonTrack.Add(atom.Key.ToString(), track.getItemInfoByAtom(atom.Value));
						}
						jsonTracks.Add(jsonTrack);
					}
				}
				json.Add("tracksArray", jsonTracks);
				json.Add(context.Request.Uri.AbsolutePath, true.ToString());
				return OutputData.Output(context, json);
			}
			catch (HttpException ex)
			{
				throw ex;
			}
			catch (Exception ex)
			{
				return OutputException.Output(context, ex);
			}
		}
	}
}